package com.yunfeiyun.agmp.common.web.system.service.impl;

import com.yunfeiyun.agmp.common.constant.CacheConstants;
import com.yunfeiyun.agmp.common.constant.ErrorCode;
import com.yunfeiyun.agmp.common.constant.UserConstants;
import com.yunfeiyun.agmp.common.exception.BizException;
import com.yunfeiyun.agmp.common.web.system.domain.SysConfig;
import com.yunfeiyun.agmp.common.web.system.mapper.SysConfigMapper;
import com.yunfeiyun.agmp.common.web.system.service.ISysConfigService;
import com.yunfeiyun.agmp.common.annotation.DataSource;
import com.yunfeiyun.agmp.common.core.text.Convert;
import com.yunfeiyun.agmp.common.enums.DataSourceType;
import com.yunfeiyun.agmp.common.enums.RedisCacheKey;
import com.yunfeiyun.agmp.common.framework.manager.RedisCacheManager;
import com.yunfeiyun.agmp.common.utils.ConfigUtils;
import com.yunfeiyun.agmp.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import javax.annotation.PostConstruct;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;

/**
 * 参数配置 服务层实现
 * 
 * @author 杨晓辉
 */
@Service
public class SysConfigServiceImpl implements ISysConfigService
{
    @Autowired
    private SysConfigMapper configMapper;

    @Autowired
    private RedisCacheManager redisCacheManager;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init()
    {
        loadingConfigCache();
    }

    /**
     * 查询参数配置信息
     * 
     * @param configId 参数配置ID
     * @return 参数配置信息
     */
    @Override
    @DataSource(DataSourceType.MASTER)
    public SysConfig selectConfigById(String configId)
    {
        SysConfig config = new SysConfig();
        config.setConfigId(configId);
        return configMapper.selectConfig(config);
    }

    /**
     * 根据键名查询参数配置信息
     * 
     * @param configKey 参数key
     * @return 参数键值
     */
    @Override
    public String selectConfigByKey(String configKey)
    {
        String configValue = Convert.toStr(redisCacheManager.getCacheObject(RedisCacheKey.SYS_CONFIG,configKey));
        if (StringUtils.isNotEmpty(configValue))
        {
            return configValue;
        }
        SysConfig config = new SysConfig();
        config.setConfigKey(configKey);
        SysConfig retConfig = configMapper.selectConfig(config);
        if (StringUtils.isNotNull(retConfig))
        {
            redisCacheManager.setCacheObject(RedisCacheKey.SYS_CONFIG,configKey, retConfig.getConfigValue());
            return retConfig.getConfigValue();
        }
        return StringUtils.EMPTY;
    }

    /**
     * 根据键名查询参数配置信息
     *
     * @param configKey 参数key
     * @return 参数键值
     */
    @Override
    public SysConfig selectConfigByKey(String configKey,int type)
    {
        SysConfig config = new SysConfig();
        config.setConfigKey(configKey);
        String configValue = Convert.toStr(redisCacheManager.getCacheObject(RedisCacheKey.SYS_CONFIG,configKey));
        if (StringUtils.isNotEmpty(configValue))
        {
            config.setConfigValue(configValue);
        }
        SysConfig retConfig = configMapper.selectConfig(config);
        if (StringUtils.isNotNull(retConfig))
        {
            redisCacheManager.setCacheObject(RedisCacheKey.SYS_CONFIG,configKey, retConfig.getConfigValue());
            config.setConfigValue(configValue);
            config.setConfigId( retConfig.getConfigId() );
        }
        return config;
    }

    /**
     * 获取验证码开关
     * 
     * @return true开启，false关闭
     */
    @Override
    public boolean selectCaptchaEnabled()
    {
        String captchaEnabled = selectConfigByKey(ConfigUtils.ConfigKeyEnum.CAPTCHA_ENABLED.getKey());
        if (StringUtils.isEmpty(captchaEnabled))
        {
            return true;
        }
        return Convert.toBool(captchaEnabled);
    }

    /**
     * 查询参数配置列表
     * 
     * @param config 参数配置信息
     * @return 参数配置集合
     */
    @Override
    public List<SysConfig> selectConfigList(SysConfig config)
    {
        return configMapper.selectConfigList(config);
    }

    /**
     * 新增参数配置
     * 
     * @param config 参数配置信息
     * @return 结果
     */
    @Override
    public int insertConfig(SysConfig config)
    {
        config.setConfigId(config.getId());
        int row = configMapper.insertConfig(config);
        if (row > 0)
        {
            redisCacheManager.setCacheObject(RedisCacheKey.SYS_CONFIG,config.getConfigKey(), config.getConfigValue());
        }
        return row;
    }

    /**
     * 修改参数配置
     * 
     * @param config 参数配置信息
     * @return 结果
     */
    @Override
    public int updateConfig(SysConfig config)
    {
        SysConfig temp = configMapper.selectConfigById(config.getConfigId());
        if( temp == null ){
            redisCacheManager.deleteObject(RedisCacheKey.SYS_CONFIG,config.getConfigKey());
            int row = configMapper.insertConfig(config);
        }
        if (StringUtils.equals(temp.getConfigKey(), config.getConfigKey()))
        {
            redisCacheManager.deleteObject(RedisCacheKey.SYS_CONFIG,temp.getConfigKey());
        }
        int row = configMapper.updateConfig(config);
        if (row > 0)
        {
            redisCacheManager.setCacheObject(RedisCacheKey.SYS_CONFIG,config.getConfigKey(), config.getConfigValue());
        }
        return row;
    }

    /**
     * 批量删除参数信息
     * 
     * @param configIds 需要删除的参数ID
     */
    @Override
    public void deleteConfigByIds(String[] configIds)
    {
        for (String configId : configIds)
        {
            SysConfig config = selectConfigById(configId);
            if (StringUtils.equals(UserConstants.YES, config.getConfigType()))
            {
                throw new BizException(ErrorCode.ACCESS_FORBIDDEN.getCode(),String.format("内置参数【%1$s】不能删除 ", config.getConfigKey()));
            }
            configMapper.deleteConfigById(configId);
            redisCacheManager.deleteObject(RedisCacheKey.SYS_CONFIG,config.getConfigKey());
        }
    }

    /**
     * 加载参数缓存数据
     */
    @Override
    public void loadingConfigCache()
    {
        List<SysConfig> configsList = configMapper.selectConfigList(new SysConfig());
        for (SysConfig config : configsList)
        {
            redisCacheManager.setCacheObject(RedisCacheKey.SYS_CONFIG,config.getConfigKey(), config.getConfigValue());
        }
    }

    /**
     * 清空参数缓存数据
     */
    @Override
    public void clearConfigCache()
    {
        Collection<String> keys = redisCacheManager.keys(CacheConstants.SYS_CONFIG_KEY + "*");
        HashMap<RedisCacheKey,Collection<String>> keyListHashMap = new HashMap<>();
        keyListHashMap.put(RedisCacheKey.SYS_CONFIG,keys);
        redisCacheManager.deleteObject(keyListHashMap);
    }

    /**
     * 重置参数缓存数据
     */
    @Override
    public void resetConfigCache()
    {
        clearConfigCache();
        loadingConfigCache();
    }

    /**
     * 校验参数键名是否唯一
     * 
     * @param config 参数配置信息
     * @return 结果
     */
    @Override
    public boolean checkConfigKeyUnique(SysConfig config)
    {
        String configId = StringUtils.isNull(config.getConfigId()) ? "-1" : config.getConfigId();
        SysConfig info = configMapper.checkConfigKeyUnique(config.getConfigKey());
        if (StringUtils.isNotNull(info) && !info.getConfigId().equalsIgnoreCase(configId))
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 设置cache key
     * 
     * @param configKey 参数键
     * @return 缓存键key
     */
    private String getCacheKey(String configKey)
    {
        return CacheConstants.SYS_CONFIG_KEY + configKey;
    }
}
